import jax
import jax.numpy as jnp
import haiku as hk 

'''
cf fig1b of https://pubs.acs.org/doi/10.1021/acs.jctc.0c00580
'''
def make_network(depth, channels, kernelsize):

    def forward_fn(x):
        x = jnp.expand_dims(x, axis=1)
        for _ in range(depth):
            resblock = hk.Sequential([
                   hk.Conv1D(output_channels=channels, kernel_shape=kernelsize, padding='SAME',with_bias=False),
                   jax.nn.silu, 
                   hk.Conv1D(output_channels=channels, kernel_shape=kernelsize, padding='SAME',with_bias=False),
            ])
            x = jax.nn.silu(x + resblock(x))
        x = hk.Conv1D(output_channels=1, kernel_shape=kernelsize, padding='SAME',with_bias=False)(x).squeeze(axis=1)
        return jax.nn.silu(x)

    return hk.transform(forward_fn)
